@@ -29,7 +29,7 @@ import requests_cache
 __author__ = 'Marius Gedminas <marius@gedmin.as>'
 __licence__ = 'MIT'
 __url__ = 'https://github.com/mgedmin/ghcloneall'
-__version__ = '1.10.0'
+__version__ = '1.11.0'
 
 
 CONFIG_FILE = '.ghcloneallrc'
@@ -347,11 +347,12 @@ class Progress(object):
 
 
 class Repo(object):
-    def __init__(self, name, clone_url, alt_urls=()):
+    def __init__(self, name, clone_url, alt_urls=(), default_branch='master'):
         self.name = name
         self.clone_url = clone_url
         self.urls = {clone_url}
         self.urls.update(alt_urls)
+        self.default_branch = default_branch
 
     def __repr__(self):
         return 'Repo({!r}, {!r}, {{{}}})'.format(
@@ -374,7 +375,8 @@ class Repo(object):
         # use repo['git_url'] for anonymous checkouts, but they're slower
         # (at least as long as you use SSH connection multiplexing)
         clone_url = repo['ssh_url']
-        return cls(repo['name'], clone_url, (repo['clone_url'],))
+        return cls(repo['name'], clone_url, (repo['clone_url'],),
+                   repo['default_branch'])
 
     @classmethod
     def from_gist(cls, gist):
@@ -396,6 +398,7 @@ class RepoWrangler(object):
         self.lock = threading.Lock()
 
         self.session = requests.Session()
+        self.has_auth_token = bool(token)
         if token:
             self.session.auth = ('', token)
 
@@ -420,20 +423,18 @@ class RepoWrangler(object):
         # - exclude private gists (if g['public'])
         return sorted(map(Repo.from_gist, gists), key=attrgetter('name'))
 
+    def _verify_user_token(self, user):
+        # Verify that the user and token match
+        user_data, _ = get_json_and_links('https://api.github.com/user',
+                                          session=self.session)
+        if user_data.get('login') == user:
+            return
+        raise Error('The github_user specified ({}) '
+                    'does not match the token used.'.format(user))
+
     def list_repos(self, user=None, organization=None, pattern=None,
                    include_archived=False, include_forks=False,
                    include_private=True, include_disabled=True):
-        if organization and not user:
-            owner = organization
-            list_url = 'https://api.github.com/orgs/{}/repos'.format(owner)
-        elif user and not organization:
-            owner = user
-            list_url = 'https://api.github.com/users/{}/repos'.format(owner)
-        else:
-            raise ValueError('specify either user or organization, not both')
-
-        message = "Fetching list of {}'s repositories from GitHub...".format(
-            owner)
 
         # User repositories default to sort=full_name, org repositories default
         # to sort=created.  In theory we don't care because we will sort the
@@ -442,7 +443,30 @@ class RepoWrangler(object):
         # happen before pagination, i.e. on the server side, as I want to
         # process the repositories alphabetically (both for aesthetic reasons,
         # and in order for --start-from to be useful).
-        list_url += '?sort=full_name'
+
+        if organization and not user:
+            owner = organization
+            list_url = ('https://api.github.com/orgs/{}/repos'
+                        '?sort=full_name').format(
+                            owner)
+        elif user and not organization:
+            owner = user
+            if include_private and self.has_auth_token:
+                self._verify_user_token(user)
+                # users/$name/repos does not include private repos, so
+                # we have to query for the repos owned by the current
+                # user instead.  This only works if the current token
+                # is associated with that user.
+                list_url = ('https://api.github.com/user/repos'
+                            '?affiliation=owner&sort=full_name')
+            else:
+                list_url = ('https://api.github.com/users/{}/repos'
+                            '?sort=full_name').format(owner)
+        else:
+            raise ValueError('specify either user or organization, not both')
+
+        message = "Fetching list of {}'s repositories from GitHub...".format(
+            owner)
 
         repos = self.get_github_list(list_url, message)
         if not include_archived:
@@ -618,8 +642,9 @@ class RepoTask(object):
             self.progress_item.update(' (local commits)')
             self.dirty = True
         branch = self.get_current_branch(dir)
-        if branch != 'master':
-            self.progress_item.update(' (not on master)')
+        if branch != repo.default_branch:
+            self.progress_item.update(' (not on {})'.format(
+                repo.default_branch))
             if self.options.verbose >= 2:
                 self.progress_item.extra_info('branch: {}'.format(branch))
             self.dirty = True
@@ -809,7 +834,8 @@ def _main():
         help='exclude archived repositories (default)')
     parser.add_argument(
         '--include-private', action='store_true', default=None,
-        help='include private repositories (default)')
+        help=('include private repositories '
+              '(default when a github token is provided)'))
     parser.add_argument(
         '--exclude-private', action='store_false', dest='include_private',
         help='exclude private repositories')
@@ -829,7 +855,8 @@ def _main():
         help='create a {} from command-line arguments'.format(CONFIG_FILE))
     parser.add_argument(
         '--http-cache', default='.httpcache', metavar='DBNAME',
-        # .sqlite will be appended automatically
+        # .sqlite will be appended automatically by requests-cache < 0.6
+        # requests-cache >= 0.6 adds .sqlite only when the name has no .
         help='cache HTTP requests on disk in an sqlite database for 5 minutes'
              ' (default: .httpcache)')
     parser.add_argument(
@@ -913,8 +940,12 @@ def _main():
                     CONFIG_FILE))
         return
 
+    if args.include_private and not args.github_token:
+        print('Warning: Listing private repositories requires a GitHub token',
+              file=sys.stderr)
+        args.include_private = False
     if args.include_private is None:
-        args.include_private = True
+        args.include_private = bool(args.github_token)
     if args.include_disabled is None:
         args.include_disabled = True
 
